searchData={"items":[{"type":"behaviour","title":"NimbleCSV","doc":"NimbleCSV is a small and fast parsing and dumping library.\n\nIt works by building highly-inlined CSV parsers, designed\nto work with strings, enumerables, and streams. At the top\nof your file (and not inside a function), you can define your\nown parser module:\n\n    NimbleCSV.define(MyParser, separator: \"\\t\", escape: \"\\\"\")\n\nOnce defined, we can parse data accordingly:\n\n    iex> MyParser.parse_string(\"name\\tage\\njohn\\t27\")\n    [[\"john\",\"27\"]]\n\nSee the `define/2` function for the list of functions that\nwould be defined in `MyParser`.","ref":"NimbleCSV.html"},{"type":"behaviour","title":"Parsing - NimbleCSV","doc":"NimbleCSV is by definition restricted in scope to do only\nparsing (and dumping). The example above discarded the\nheaders when parsing the string, as NimbleCSV expects\ndevelopers to handle those explicitly later.\nFor example:\n\n    \"name\\tage\\njohn\\t27\"\n    |> MyParser.parse_string()\n    |> Enum.map(fn [name, age] ->\n      %{name: name, age: String.to_integer(age)}\n    end)\n\nThis is particularly useful with the `c:parse_stream/1` functionality\nthat receives and returns a stream. For example, we can use it\nto parse files line by line lazily:\n\n    \"path/to/csv/file\"\n    |> File.stream!(read_ahead: 100_000)\n    |> MyParser.parse_stream()\n    |> Stream.map(fn [name, age] ->\n      %{name: :binary.copy(name), age: String.to_integer(age)}\n    end)\n    |> Stream.run()\n\nBy default this library ships with two implementations:\n\n  * `NimbleCSV.RFC4180`, which is the most common implementation of\n    CSV parsing/dumping available using comma as separators and\n    double-quote as escape. If you want to use it in your codebase,\n    simply alias it to CSV and enjoy:\n\n        iex> alias NimbleCSV.RFC4180, as: CSV\n        iex> CSV.parse_string(\"name,age\\njohn,27\")\n        [[\"john\",\"27\"]]\n\n  * `NimbleCSV.Spreadsheet`, which uses UTF-16 and is most commonly\n    used by spreadsheet software, such as Excel, Numbers, etc.","ref":"NimbleCSV.html#module-parsing"},{"type":"behaviour","title":"Binary references - NimbleCSV","doc":"One of the reasons behind NimbleCSV performance is that it performs\nparsing by matching on binaries and extracting those fields as\nbinary references. Therefore, if you have a row such as:\n\n    one,two,three,four,five\n\nNimbleCSV will return a list of `[\"one\", \"two\", \"three\", \"four\", \"five\"]`\nwhere each element references the original row. For this reason, if\nyou plan to keep the parsed data around in the parsing process or even\nsend it to another process, you must copy the data before doing the transfer,\nthat's why we use `:binary.copy/1` in the examples above.","ref":"NimbleCSV.html#module-binary-references"},{"type":"behaviour","title":"Dumping - NimbleCSV","doc":"NimbleCSV can dump any enumerable to either iodata or to streams:\n\n    iex> IO.iodata_to_binary MyParser.dump_to_iodata([~w(name age), ~w(mary 28)])\n    \"name\\tage\\nmary\\t28\\n\"\n\n    iex> MyParser.dump_to_stream([~w(name age), ~w(mary 28)])\n    #Stream<[\n      enum: [[\"name\", \"age\"], [\"mary\", \"28\"]],\n      funs: [#Function<47.127921642/1 in Stream.map/2>]\n    ]>","ref":"NimbleCSV.html#module-dumping"},{"type":"function","title":"NimbleCSV.define/2","doc":"Defines a new parser/dumper.\n\nCalling this function defines a CSV module. Therefore, `define/2`\nis typically invoked at the top of your files and not inside\nfunctions. Placing it inside a function would cause the same\nmodule to be defined multiple times, one time per invocation,\nleading your code to emit warnings and slowing down execution.\n\nIt accepts the following options:\n\n  * `:moduledoc` - the documentation for the generated module\n\nThe following options control parsing:\n\n  * `:escape`- the CSV escape, defaults to `\"\\\"\"`\n  * `:encoding` - converts the given data from encoding to UTF-8\n  * `:separator`- the CSV separators, defaults to `\",\"`. It can be\n    a string or a list of strings. If a list is given, the first entry\n    is used for dumping (see below)\n  * `:newlines` - the list of entries to be considered newlines\n    when parsing, defaults to `[\"\\r\\n\", \"\\n\"]` (note they are attempted\n    in order, so the order matters)\n  * `:trim_bom` - automatically trims\n    [BOM (byte-order marker)](https://en.wikipedia.org/wiki/Byte_order_mark)\n    when parsing string. Note the BOM is not trimmed for enumerables or streams.\n    In such cases, the BOM must be trimmed directly in the stream, such as\n    `File.stream!(path, [:trim_bom])`\n\nThe following options control dumping:\n\n  * `:escape`- the CSV escape character, defaults to `\"\\\"\"`\n  * `:encoding` - converts the given data from UTF-8 to the given encoding\n  * `:separator`- the CSV separator character, defaults to `\",\"`\n  * `:line_separator` - the CSV line separator character, defaults to `\"\\n\"`\n  * `:dump_bom` - includes BOM (byte order marker) in the dumped document\n  * `:reserved` - the list of characters to be escaped, defaults to the\n    `:separator`, `:newlines`, and `:escape` characters above\n  * `:escape_formula` - the formula prefix(es) and formula escape sequence,\n     defaults to `nil`, which disabled formula escaping\n     `%{[\"@\", \"+\", \"-\", \"=\", \"\\t\", \"\\r\"] => \"'\"}` would escape all fields starting\n     with `@`, `+`, `-`, `=`, tab or carriage return using the `'` character.\n\nAlthough parsing may support multiple newline delimiters, when\ndumping, only one of them must be picked, which is controlled by\nthe `:line_separator` option. This allows NimbleCSV to handle both\n`\"\\r\\n\"` and `\"\\n\"` when parsing, but only the latter for dumping.","ref":"NimbleCSV.html#define/2"},{"type":"function","title":"Parser/Dumper API - NimbleCSV.define/2","doc":"Modules defined with `define/2` implement the `NimbleCSV` behaviour. See\nthe callbacks for this behaviour for information on the generated functions\nand their documentation.","ref":"NimbleCSV.html#define/2-parser-dumper-api"},{"type":"function","title":"CSV Injection - NimbleCSV.define/2","doc":"By default, the dumper does not escape values which some clients may interpret\nas formulas or commands. This can result in\n[CSV injection](https://owasp.org/www-community/attacks/CSV_Injection).\n\nThere is no universally correct way to handle CSV injections. In some cases,\nyou may want formulas to be preserved: you may want a cell to have a value of\n`=SUM(...)`. The only way to escape these values is by materially changing\nthem by prefixing a tab or single quote, which can also lead to false positives.\n\nThe `escape_formula` option will add a prefix to any value which has the\nconfigured prefix (e.g. it will prepend `'` to any value which begins with\n`@`, `+`, `-`, `=`, tab or carriage return). Use the following config to\nfollow the [OWASP recommendations](https://owasp.org/www-community/attacks/CSV_Injection):\n\n    escape_formula: %{[\"@\", \"+\", \"-\", \"=\", \"\\t\", \"\\r\"] => \"'\"}\n\nApplications that want more control over this process, to allow formulas in specific\ncases, or possibly minimize false positives, should leave this option disabled and\nescape the value, as necessary, within their code.","ref":"NimbleCSV.html#define/2-csv-injection"},{"type":"callback","title":"NimbleCSV.dump_to_iodata/1","doc":"Eagerly dumps an enumerable into iodata (a list of binaries and bytes and other lists).","ref":"NimbleCSV.html#c:dump_to_iodata/1"},{"type":"callback","title":"NimbleCSV.dump_to_stream/1","doc":"Lazily dumps from an enumerable to a stream.\n\nIt returns a stream that emits each row as iodata.","ref":"NimbleCSV.html#c:dump_to_stream/1"},{"type":"callback","title":"NimbleCSV.parse_enumerable/1","doc":"Same as `parse_enumerable(enumerable, [])`.","ref":"NimbleCSV.html#c:parse_enumerable/1"},{"type":"callback","title":"NimbleCSV.parse_enumerable/2","doc":"Eagerly parses CSV from an enumerable and returns a list of rows.\n\nRaises `NimbleCSV.ParseError` for an invalid CSV.","ref":"NimbleCSV.html#c:parse_enumerable/2"},{"type":"callback","title":"Options - NimbleCSV.parse_enumerable/2","doc":"* `:skip_headers` - when `true`, skips headers. Defaults to `true`.\n    Set it to `false` to keep headers or when the CSV has no headers.","ref":"NimbleCSV.html#c:parse_enumerable/2-options"},{"type":"callback","title":"NimbleCSV.parse_stream/1","doc":"Same as `parse_stream(enumerable, [])`.","ref":"NimbleCSV.html#c:parse_stream/1"},{"type":"callback","title":"NimbleCSV.parse_stream/2","doc":"Lazily parses CSV from a stream and returns a stream of rows.\n\nIt expects the given enumerable to be line-oriented, where each\nentry in the enumerable is a line. If your stream does not conform\nto that, you can call `c:to_line_stream/1` before parsing the stream.\n\nRaises `NimbleCSV.ParseError` for an invalid CSV.","ref":"NimbleCSV.html#c:parse_stream/2"},{"type":"callback","title":"Options - NimbleCSV.parse_stream/2","doc":"* `:skip_headers` - when `true`, skips headers. Defaults to `true`.\n    Set it to `false` to keep headers or when the CSV has no headers.","ref":"NimbleCSV.html#c:parse_stream/2-options"},{"type":"callback","title":"NimbleCSV.parse_string/1","doc":"Same as `parse_string(enumerable, [])`.","ref":"NimbleCSV.html#c:parse_string/1"},{"type":"callback","title":"NimbleCSV.parse_string/2","doc":"Eagerly parses CSV from a string and returns a list of rows.\n\nRaises `NimbleCSV.ParseError` for an invalid CSV.","ref":"NimbleCSV.html#c:parse_string/2"},{"type":"callback","title":"Options - NimbleCSV.parse_string/2","doc":"* `:skip_headers` - when `true`, skips headers. Defaults to `true`.\n    Set it to `false` to keep headers or when the CSV has no headers.","ref":"NimbleCSV.html#c:parse_string/2-options"},{"type":"callback","title":"NimbleCSV.to_line_stream/1","doc":"Lazily convert a stream of arbitrarily chunked binaries to a line-oriented one.\n\nThis is useful for places where a CSV cannot be streamed in a\nline-oriented fashion from its source.","ref":"NimbleCSV.html#c:to_line_stream/1"},{"type":"module","title":"NimbleCSV.RFC4180","doc":"A CSV parser that uses comma as separator and double-quotes as escape according to RFC4180.","ref":"NimbleCSV.RFC4180.html"},{"type":"function","title":"NimbleCSV.RFC4180.dump_to_iodata/1","doc":"Callback implementation for  NimbleCSV.dump_to_iodata/1 .","ref":"NimbleCSV.RFC4180.html#dump_to_iodata/1"},{"type":"function","title":"NimbleCSV.RFC4180.dump_to_stream/1","doc":"Callback implementation for  NimbleCSV.dump_to_stream/1 .","ref":"NimbleCSV.RFC4180.html#dump_to_stream/1"},{"type":"function","title":"NimbleCSV.RFC4180.parse_enumerable/2","doc":"Callback implementation for  NimbleCSV.parse_enumerable/2 .","ref":"NimbleCSV.RFC4180.html#parse_enumerable/2"},{"type":"function","title":"NimbleCSV.RFC4180.parse_stream/2","doc":"Callback implementation for  NimbleCSV.parse_stream/2 .","ref":"NimbleCSV.RFC4180.html#parse_stream/2"},{"type":"function","title":"NimbleCSV.RFC4180.parse_string/2","doc":"Callback implementation for  NimbleCSV.parse_string/2 .","ref":"NimbleCSV.RFC4180.html#parse_string/2"},{"type":"function","title":"NimbleCSV.RFC4180.to_line_stream/1","doc":"Callback implementation for  NimbleCSV.to_line_stream/1 .","ref":"NimbleCSV.RFC4180.html#to_line_stream/1"},{"type":"module","title":"NimbleCSV.Spreadsheet","doc":"A parser with spreadsheet friendly settings.\n\nThe parser uses tab as separator and double-quotes as escape, as required by\ncommon spreadsheet software such as Excel, Numbers and OpenOffice. It's encoded\nin UTF-16 little-endian with a byte-order BOM.\n\nSuch files should still be saved with the `.csv` extension.","ref":"NimbleCSV.Spreadsheet.html"},{"type":"function","title":"NimbleCSV.Spreadsheet.dump_to_iodata/1","doc":"Callback implementation for  NimbleCSV.dump_to_iodata/1 .","ref":"NimbleCSV.Spreadsheet.html#dump_to_iodata/1"},{"type":"function","title":"NimbleCSV.Spreadsheet.dump_to_stream/1","doc":"Callback implementation for  NimbleCSV.dump_to_stream/1 .","ref":"NimbleCSV.Spreadsheet.html#dump_to_stream/1"},{"type":"function","title":"NimbleCSV.Spreadsheet.parse_enumerable/2","doc":"Callback implementation for  NimbleCSV.parse_enumerable/2 .","ref":"NimbleCSV.Spreadsheet.html#parse_enumerable/2"},{"type":"function","title":"NimbleCSV.Spreadsheet.parse_stream/2","doc":"Callback implementation for  NimbleCSV.parse_stream/2 .","ref":"NimbleCSV.Spreadsheet.html#parse_stream/2"},{"type":"function","title":"NimbleCSV.Spreadsheet.parse_string/2","doc":"Callback implementation for  NimbleCSV.parse_string/2 .","ref":"NimbleCSV.Spreadsheet.html#parse_string/2"},{"type":"function","title":"NimbleCSV.Spreadsheet.to_line_stream/1","doc":"Callback implementation for  NimbleCSV.to_line_stream/1 .","ref":"NimbleCSV.Spreadsheet.html#to_line_stream/1"},{"type":"exception","title":"NimbleCSV.ParseError","doc":"","ref":"NimbleCSV.ParseError.html"},{"type":"extras","title":"Changelog for NimbleCSV","doc":"# Changelog for NimbleCSV","ref":"changelog.html"},{"type":"extras","title":"v1.3.0 (2025-06-24) - Changelog for NimbleCSV","doc":"* Require Elixir 1.15+\n  * Add `generated: true` to `newlines_separator` macro\n  * Fix warnings on Elixir 1.20+\n  * Document OWASP official recommendations for CSV injections","ref":"changelog.html#v1-3-0-2025-06-24"},{"type":"extras","title":"v1.2.0 (2021-12-17) - Changelog for NimbleCSV","doc":"* Add `to_line_stream/1` for converting streams into line-oriented ones\n  * Support the `:escape_formula` option to prevent code injection via CSV","ref":"changelog.html#v1-2-0-2021-12-17"},{"type":"extras","title":"v1.1.0 (2020-09-30) - Changelog for NimbleCSV","doc":"* Fix CSV dumper to use \\r\\n as the line separator according to the spec\n  * Automatically include new lines in the list of reserved characters","ref":"changelog.html#v1-1-0-2020-09-30"},{"type":"extras","title":"v1.0.0 (2020-09-25) - Changelog for NimbleCSV","doc":"* First stable release","ref":"changelog.html#v1-0-0-2020-09-25"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}